// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_1.x.x_GSC_DN/samples/sbtest/osc_test.c $
// $Rev: 53081 $
// $Date: 2023-06-13 10:10:10 -0500 (Tue, 13 Jun 2023) $

// SIO4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	TEST_CHIP(w,o,l)		_value_test("chip",(w),(o)->chip,(l))
#define	TEST_FREQ_GOT(w,o,l)	_value_test("freq_got",(w),(o)->freq_got,(l))
#define	TEST_FREQ_REF(w,o,l)	_value_test("freq_ref",(w),(o)->freq_ref,(l))
#define	TEST_FREQ_WANT(w,o,l)	_value_test("freq_want",(w),(o)->freq_want,(l))



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int _value_test(const char* item, u32 want, u32 got, int line)
{
	int	errs	= 0;

	if (got != want)
	{
		errs++;
		printf(	"FAIL <---  (%d. %s: got %lu, expected %lu)\n",
				line,
				item,
				(long) got,
				(long) want);
	}

	return(errs);
}



//*****************************************************************************
static int _reference_set(int fd, u32 want, int line)
{
	int			errs	= 0;
	sio4_osc_t	osc;
	int			ret;

	memset(&osc, 0, sizeof(osc));
	osc.freq_ref	= want;
	ret				= sio4_ioctl(fd, SIO4_OSC_REFERENCE, &osc);

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. SIO4_OSC_REFERENCE)\n",
				line);
	}
	else if (osc.freq_ref != want)
	{
		errs++;
		printf(	"FAIL <---  (%d. SIO4_OSC_REFERENCE, expected %ld, got %ld)\n",
				line,
				(long) want,
				(long) osc.freq_ref);
	}

	return(errs);
}



//*****************************************************************************
static int _osc_test_test(int fd, const data_t* data, s32 ref)
{
	s32			delta		= 0;
	int			errs		= 0;
	s32			freq_got	= 0;
	s32			freq_ref	= 0;
	s32			freq_want;
	s32			max			= 0;
	s32			min			= 0;
	long		mmax;
	long		mmin;
	sio4_osc_t	osc;
	int			ret;

	errs	+= sio4_reset_channel(fd, 0);

	switch (data->osc_chip)
	{
		default:

			errs++;
			printf("FAIL <---  (%d. internal error)\n", __LINE__);
			break;

		case SIO4_OSC_CHIP_FIXED:

			freq_ref	= 20000000L;
			min			= 1000000L;
			max			= 20000000L;
			delta		= 1000000L;
			break;

		case SIO4_OSC_CHIP_CY22393:
		case SIO4_OSC_CHIP_CY22393_2:

			freq_ref	= ref;
			min			= 1000000L;
			max			= 20000000L;
			delta		= 1000000L;
			break;
	}

	for (freq_want = min; (errs == 0) && (freq_want <= max); freq_want += delta)
	{
		errs	+= _reference_set(fd, freq_ref, __LINE__);
		memset(&osc, 0, sizeof(osc));
		osc.freq_want	= freq_want;
		ret		= sio4_ioctl(fd, SIO4_OSC_TEST, &osc);

		if (ret < 0)
		{
			errs++;
			printf("FAIL <---  (%d. SIO4_OSC_TEST)\n", __LINE__);
			break;
		}

		switch (data->osc_chip)
		{
			default:

				errs++;
				printf("FAIL <---  (%d. internal error)\n", __LINE__);
				break;

			case SIO4_OSC_CHIP_FIXED:

				freq_got	= freq_ref;
				break;

			case SIO4_OSC_CHIP_CY22393:
			case SIO4_OSC_CHIP_CY22393_2:

				freq_got	= freq_want;
				// This won't pass every conceivable circumstance, but it will most instances.
				mmax	= ((osc.freq_want / 10) + 1 + ((osc.freq_want + 9999) / 10000)) * 10 + 100;
				mmin	= ((osc.freq_want / 10) - 1 - ((osc.freq_want + 9999) / 10000)) * 10 - 100;

				if (osc.freq_got < mmin)
				{
//					printf(	"\n(%d. SMALL: W %ld, G %ld, Ref %ld, mmin, %ld)",
//							__LINE__, (long) osc.freq_want, (long) osc.freq_got, (long) freq_ref, (long) mmin);
				}
				else if (osc.freq_got > mmax)
				{
//					printf(	"\n(%d. LARGE: W %ld, G %ld, Ref %ld, mmax %ld)",
//							__LINE__, (long) osc.freq_want, (long) osc.freq_got, (long) freq_ref, (long) mmax);
				}
				else
				{
					osc.freq_got	= osc.freq_want;
				}

				break;
		}

		errs	+= TEST_FREQ_GOT(freq_got, &osc, __LINE__);
		errs	+= TEST_FREQ_WANT(freq_want, &osc, __LINE__);
		errs	+= TEST_FREQ_REF(freq_ref, &osc, __LINE__);
		errs	+= TEST_CHIP(data->osc_chip, &osc, __LINE__);
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd, const data_t* data, s32 ref)
{
	int	errs	= 0;

	errs	+= _osc_test_test(fd, data, ref);
	return(errs);
}



//*****************************************************************************
int osc_test_tests(int fd, const data_t* data)
{
	int	errs	= 0;
	s32	ref		= SIO4_OSC_FREQ_REF_DEFAULT;

	gsc_label("SIO4_OSC_TEST");

	if (data->osc_measure)
		errs	+= sio4_osc_reference(fd, &ref, 0);

	if (errs == 0)
	{
		switch (data->osc_chip)
		{
			default:

				printf(	"SKIPPED  (Unrecognized oscillator option: %ld)\n",
						(long) data->osc_chip);
				break;

			case SIO4_OSC_CHIP_UNKNOWN:

				errs++;
				printf("FAIL <---  (Oscillator chip not recognized.)\n");
				break;

			case SIO4_OSC_CHIP_IDC2053B:
			case SIO4_OSC_CHIP_IDC2053B_4:

				printf("SKIPPED  (Oscillator chip not supported by test.)\n");
				break;

			case SIO4_OSC_CHIP_FIXED:
			case SIO4_OSC_CHIP_CY22393:
			case SIO4_OSC_CHIP_CY22393_2:

				errs	+= _service_test(fd);
				errs	+= _function_test(fd, data, ref);

				if (errs == 0)
					printf("PASS\n");

				break;
		}
	}

	errs	= errs ? 1 : 0;
	return(errs);
}


